import java.io.*;
import java.security.*;
import xjava.security.*;
import cryptix.provider.key.*;
import cryptix.provider.rsa.*;
import cryptix.util.core.Hex;


public class Crypto {

    public  boolean DEBUG = false;

    public  static SecureRandom rand = new SecureRandom();

    public Crypto() {
	java.security.Security.addProvider(new cryptix.provider.Cryptix());
    }

    private  void debug(String s) {
	if (DEBUG) {
	    System.out.println(s);
	}
    }

    public synchronized  Key makeRijndaelKey() {
	RijndaelKeyGenerator rkg = new RijndaelKeyGenerator();
	rkg.initialize(rand);
	return rkg.generateKey();
    }

    public synchronized  byte[] objToBytes(Serializable obj) 
	throws IOException
    {
	ByteArrayOutputStream buf = new ByteArrayOutputStream();
	ObjectOutputStream oos = new ObjectOutputStream(buf);
	oos.writeObject(obj);
	return buf.toByteArray();
    }

    public synchronized  byte[] encryptRSA(Serializable obj, PublicKey kPub) 
	throws KeyException, IOException
    {
	try {
	    Cipher RSACipher = Cipher.getInstance("RSA");
	    return encrypt(RSACipher, obj, kPub);
	} catch (NoSuchAlgorithmException e) {
	    System.exit(1);
	}
	return null;
    }

    public synchronized  byte[] encryptRijndael(Serializable obj, Key k) 
	throws KeyException, IOException
    {
	try {
	    Cipher RijndaelCipher = Cipher.getInstance("Rijndael");
	    return encrypt(RijndaelCipher, obj, k);
	} catch (NoSuchAlgorithmException e) {
	    System.exit(1);
	}
	return null;
    }

    public synchronized  Object decryptRSA(byte[] msgE, PrivateKey kPriv)
	throws KeyException, IOException
    {
	try {
	    Cipher RSACipher = Cipher.getInstance("RSA");
	    return decrypt(RSACipher, msgE, kPriv);
	} catch (NoSuchAlgorithmException e) {
	    System.exit(1);
	}
	return null;
    }

    public synchronized  Object decryptRijndael(byte[] msgE, Key k) 
	throws KeyException, IOException
    {
	try {
	    Cipher RijndaelCipher = Cipher.getInstance("Rijndael");
	    return decrypt(RijndaelCipher, msgE, k);
	} catch (NoSuchAlgorithmException e) {
	    System.exit(1);
	}
	return null;
    }

    private synchronized  byte[] encrypt(Cipher cipher, 
				  Serializable obj, 
				  Key kPub) 
	throws KeyException, IOException
    {
	try {
	    // Initialize the cipher
	    // This determines the input block size
	    debug("Initializing cipher for encryption.");
	    cipher.initEncrypt(kPub);
	    int inBlockSize = cipher.getInputBlockSize();
	    debug("cipher.inputBlockSize = " + inBlockSize);

	    // The data format is: 
	    //   srcLength bytes of data, where srcLength = bos.size
	    //   padSize bytes of padding: srcLength + k % InputBlockSize = 0
	    ByteArrayOutputStream padBuf = new ByteArrayOutputStream();
	    ObjectOutputStream oos = new ObjectOutputStream(padBuf);
	    oos.writeObject(obj);
	    int padSize = inBlockSize - (padBuf.size() % inBlockSize);
	    debug("padBuf.size() = " + padBuf.size());

	    // The pad bytes are chosen randomly to prevent known plaintext
	    // attacks.
	    byte[] padBytes = new byte[padSize];
	    rand.nextBytes(padBytes);
	    padBuf.write(padBytes, 0, padSize);

	    // srcBytes contains the final input to the cipher engine
	    byte[] srcBytes = padBuf.toByteArray();
	    debug("srcBytes = " + Hex.dumpString(srcBytes));
	    debug("Serialized size = " + srcBytes.length);	    

	    // Now encrypt each block of input data and store it in
	    // the output buffer.
	    int numBlocks = srcBytes.length / inBlockSize;
	    int outBlockSize = cipher.getOutputBlockSize();
	    debug("Number of blocks = " + numBlocks);
	    debug("outBlockSize = " + outBlockSize);

	    ByteArrayOutputStream outBuf = new ByteArrayOutputStream();
	    byte[] buf;
	    int i;
	    for (i=0; i<numBlocks; i++) {
		buf = cipher.crypt(srcBytes, i*inBlockSize, inBlockSize);
		outBuf.write(buf, 0, outBlockSize);
	    }
	    buf = outBuf.toByteArray();
	    debug("encrypted buf = " + Hex.dumpString(buf));
	    debug("Num iters = " + i);
	    debug("buf.length = " + buf.length);

	    return buf;
	} catch (IllegalBlockSizeException e) {
	    System.out.println("BlockSize: " + e.getBlockSize());
	    System.out.println("DataSize: " + e.getDataSize());
	    e.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    private synchronized  Object decrypt(Cipher cipher,
				 byte[] msgE, 
				 Key kPriv) 
	throws KeyException, IOException
    {
	try {
	    debug("Initializing cipher for decryption.");
	    cipher.initDecrypt(kPriv);

	    int inBlockSize = cipher.getInputBlockSize();
	    int outBlockSize = cipher.getOutputBlockSize();

	    debug("inBlockSize = " + inBlockSize);
	    debug("outBlockSize = " + inBlockSize);
	    debug("msgE = " + Hex.dumpString(msgE));

	    int numBlocks = msgE.length / inBlockSize;
	    debug("numBlocks = " + numBlocks);

	    int i;
	    ByteArrayOutputStream outBuf = new ByteArrayOutputStream();
	    byte[] buf;
	    for (i=0; i<numBlocks; i++) {
		debug(".");
		buf = cipher.crypt(msgE, i*inBlockSize, inBlockSize);
		outBuf.write(buf, 0, outBlockSize);
	    }
	    buf = outBuf.toByteArray();
	    debug("output = " + Hex.dumpString(buf));
	    ByteArrayInputStream bis = new ByteArrayInputStream(buf);
	    ObjectInputStream ois = new ObjectInputStream(bis);
	    return ois.readObject();
	} catch (IllegalBlockSizeException e) {
	    System.out.println("BlockSize: " + e.getBlockSize());
	    System.out.println("DataSize: " + e.getDataSize());
	    e.printStackTrace();
	    System.exit(1);
	} catch (ClassNotFoundException e) {
	    e.printStackTrace();
	    System.exit(1);
	}
	return null;
    }

    public synchronized  byte[] sign(byte[] msg, PrivateKey kPriv) 
	throws SignatureException, KeyException, IOException
    {
	// Initialize the signature object for signing
	debug("Initializing signature.");
	try {
	    Signature RSASig = Signature.getInstance("SHA-1/RSA/PKCS#1");
	    debug("Using algorithm: " + RSASig.getAlgorithm());
	    RSASig.initSign(kPriv);
	    RSASig.update(msg);
	    return RSASig.sign();
	} catch (NoSuchAlgorithmException e) {
	    System.exit(1);
	}
	return null;
    }

    public synchronized  boolean verify(byte[] msg, byte[] sig, PublicKey kPub) 
	throws SignatureException, KeyException
    {
	// Initialize the signature object for verifying
	debug("Initializing signature.");
	try {
	    Signature RSASig = Signature.getInstance("SHA-1/RSA/PKCS#1");
	    RSASig.initVerify(kPub);
	    RSASig.update(msg);
	    return RSASig.verify(sig);
	} catch (NoSuchAlgorithmException e) {
	    System.exit(1);
	}
	return false;
    }

}
